#include "bmpWriter.h"

#include <cstdint>

using Byte = uint8_t;
using Word = uint16_t;
using Dword = uint32_t;

class RawOstream {
    std::ostream &_out;
public:
    RawOstream( std::ostream &out ) :
        _out( out ) {}

    RawOstream &operator<<( const BitmapWriter::RGB &rgb ) {
        *this << rgb.red << rgb.green << rgb.blue;
        return *this;
    }

    template< typename T >
    RawOstream &operator<<( T value ) {
        _out.write( reinterpret_cast<char*>( &value ), sizeof( T ) );
        return *this;
    }
};

void BitmapWriter::writeHeader() {
    RawOstream o( _out );

    // Header
    o << Byte( 'B' ) <<
        Byte( 'M' ) <<
        Dword( _x * _y * 3 ) <<
        Dword( 0 ) <<
        Dword( 0x36 );

    // Info
    o << Dword( sizeof( Dword ) * 9 + sizeof( Word ) * 2 ) <<
        Dword( _x ) <<
        Dword( _y ) <<
        Word( 1 ) <<
        Word( 24 ) <<
        Dword( 0 ) <<
        Dword( 0 ) <<
        Dword( 0x0ec4 ) <<
        Dword( 0x0ec4 ) <<
        Dword( 0 ) <<
        Dword( 0 );
}

void BitmapWriter::open( std::string name, size_t x, size_t y ) {
    if ( _valid )
        throw std::runtime_error( "Bad usage of open function" );
    _valid = true;
    _out.open( name.c_str(), std::ios::out | std::ios::binary );
    _x = x;
    _y = y;
    _padding = ( 4 - ( _x * 3 % 4 ) ) % 4;
    writeHeader();
}

BitmapWriter &BitmapWriter::operator<<( const RGB &pixel ) {

    if ( _positionX == _x || _positionY == _y )
        throw std::out_of_range( "pixel" );

    RawOstream o( _out );
    o << pixel;

    ++_positionX;

    if ( _positionX == _x ) {
        _positionX = 0;
        ++_positionY;

        for ( size_t i = 0; i < _padding; ++i )
            o << Byte( 0 );
    }

    return *this;
}
