#pragma once

#include <vector>
#include <iostream>
#include <string>
#include <fstream>
#include <stdexcept>

class BitmapWriter {
    bool _valid;
    std::ofstream _out;
    size_t _x;
    size_t _y;
    size_t _padding;
    size_t _positionX;
    size_t _positionY;
public:
    struct RGB {
        using Byte = unsigned char;

        Byte red;
        Byte green;
        Byte blue;

        RGB( Byte r, Byte g, Byte b ) :
            red( r ),
            green( g ),
            blue( b ) {}
        RGB() = default;
    };

    BitmapWriter( std::string name, size_t x, size_t y ) :
        _valid( true ),
        _out( name.c_str(), std::ios::out | std::ios::binary ),
        _x( x ),
        _y( y ),
        _padding( (4 - ( _x * 3 % 4 ) ) % 4 ),
        _positionX( 0 ),
        _positionY( 0 )
    {
        writeHeader();
    }
    BitmapWriter() :
        _valid( false ),
        _positionX( 0 ),
        _positionY( 0 )
    {}

    void open( std::string name, size_t x, size_t y );

    BitmapWriter &operator<<( const RGB &pixel );

private:
    void writeHeader();
};


