#pragma once

#include <iostream>
#include <string>
#include <iomanip>

class ProgressBar {
    std::ostream &_out;
    int _endPoint;
    int _point;
    int _oldPoint;
public:
    ProgressBar( std::ostream &out, std::string title, int endPoint ) :
        _out( out ),
        _endPoint( endPoint ),
        _point( 0 ),
        _oldPoint( 0 )
    {
        drawTitle( title );
    }

    ProgressBar &operator++() {
        if ( _point < _endPoint ) {
            _point++;
            if ( percentage( _oldPoint ) + 2 == percentage( _point ) ) {
                _oldPoint = _point;
                _out << '#';
                if ( _point == _endPoint )
                    _out << std::endl;
                else
                    _out << std::flush;
            }
        }
        return *this;
    }
private:
    void drawTitle( const std::string &name ) {
        const char percentsLeft[] = "| 0 %";
        const char percentsRight[] = "100 % |";
        int paddingLeft = ( 50 - name.size() - 2 ) / 2;
        int paddingRight = 50 - paddingLeft - 2 - name.size();
        if ( paddingLeft < 0 )
            throw std::invalid_argument( "Too long." );


        for ( int i = 0; i < paddingLeft; ++i )
            _out << '=';
        _out << " " << name << " ";
        for ( int i = 0; i < paddingRight; ++i )
            _out << '=';
        _out << std::endl;

        _out << percentsLeft <<
            std::setw( 50 - sizeof( percentsLeft ) + 1 ) <<
            percentsRight <<
            std::endl;
    }
    int percentage( int i ) const {
        return int( 100.0 * i / _endPoint );
    }
};
